pub struct FieldElement(pub(crate) fiat_p521_tight_field_element);Expand description
Element of the secp521r1 base field used for curve coordinates.
Tuple Fields
0: fiat_p521_tight_field_elementImplementations
Sourceimpl FieldElement
impl FieldElement
const LIMBS: usize = 9
Sourcepub fn from_bytes(repr: &FieldBytes) -> CtOption<Self>
pub fn from_bytes(repr: &FieldBytes) -> CtOption<Self>
Create a FieldElement from a canonical big-endian representation.
Sourcepub fn from_slice(slice: &[u8]) -> Result<Self>
pub fn from_slice(slice: &[u8]) -> Result<Self>
Decode FieldElement from a big endian byte slice.
Sourcepub(crate) const fn from_hex(hex: &str) -> Self
pub(crate) const fn from_hex(hex: &str) -> Self
Parse a FieldElement from big endian hex-encoded bytes.
This method is primarily intended for defining internal constants.
Panics
- if the input in hex is not the correct length
- if the given value when decoded from hex overflows the modulus
Sourcepub const fn from_u64(w: u64) -> Self
pub const fn from_u64(w: u64) -> Self
Convert a u64 into a FieldElement.
Sourcepub(crate) const fn from_uint_unchecked(w: Uint) -> Self
pub(crate) const fn from_uint_unchecked(w: Uint) -> Self
Decode FieldElement from Uint.
Does not perform a check that the field element does not overflow the order.
Used incorrectly this can lead to invalid results!
Sourcepub const fn to_bytes(self) -> FieldBytes
pub const fn to_bytes(self) -> FieldBytes
Returns the big-endian encoding of this FieldElement.
Sourcepub fn is_odd(&self) -> Choice
pub fn is_odd(&self) -> Choice
Determine if this FieldElement is odd in the SEC1 sense: self mod 2 == 1.
Returns
If odd, return Choice(1). Otherwise, return Choice(0).
Sourcepub fn is_even(&self) -> Choice
pub fn is_even(&self) -> Choice
Determine if this FieldElement is even in the SEC1 sense: self mod 2 == 0.
Returns
If even, return Choice(1). Otherwise, return Choice(0).
Sourcepub fn is_zero(&self) -> Choice
pub fn is_zero(&self) -> Choice
Determine if this FieldElement is zero.
Returns
If zero, return Choice(1). Otherwise, return Choice(0).
Sourcepub const fn add_loose(&self, rhs: &Self) -> LooseFieldElement
pub const fn add_loose(&self, rhs: &Self) -> LooseFieldElement
Add elements.
Sourcepub const fn double_loose(&self) -> LooseFieldElement
pub const fn double_loose(&self) -> LooseFieldElement
Double element (add it to itself).
Sourcepub const fn sub_loose(&self, rhs: &Self) -> LooseFieldElement
pub const fn sub_loose(&self, rhs: &Self) -> LooseFieldElement
Subtract elements, returning a loose field element.
Sourcepub const fn neg_loose(&self) -> LooseFieldElement
pub const fn neg_loose(&self) -> LooseFieldElement
Negate element, returning a loose field element.
Sourcepub const fn pow_vartime<const RHS_LIMBS: usize>(
&self,
exp: &Uint<RHS_LIMBS>,
) -> Self
pub const fn pow_vartime<const RHS_LIMBS: usize>( &self, exp: &Uint<RHS_LIMBS>, ) -> Self
Returns self^exp, where exp is a little-endian integer exponent.
This operation is variable time with respect to the exponent exp.
If the exponent is fixed, this operation is constant time.
Sourcepub const fn sqn_vartime(&self, n: usize) -> Self
pub const fn sqn_vartime(&self, n: usize) -> Self
Returns self^(2^n) mod p.
This operation is variable time with respect to the exponent n.
If the exponent is fixed, this operation is constant time.
Sourcepub fn invert(&self) -> CtOption<Self>
pub fn invert(&self) -> CtOption<Self>
Compute FieldElement inversion: 1 / self.
Sourcepub fn invert_vartime(&self) -> CtOption<Self>
pub fn invert_vartime(&self) -> CtOption<Self>
Compute FieldElement inversion: 1 / self in variable-time.
Sourceconst fn invert_unwrap(&self) -> Self
const fn invert_unwrap(&self) -> Self
Sourcepub fn sqrt(&self) -> CtOption<Self>
pub fn sqrt(&self) -> CtOption<Self>
Returns the square root of self mod p, or None if no square root
exists.
Implementation details
If x has a sqrt, then due to Euler’s criterion this implies x(p - 1)/2 = 1.
- x(p + 1)/2 = x.
- There’s a special property due to p ≡ 3 (mod 4) which implies (p + 1)/4 is an integer.
- We can rewrite
1.as x((p+1)/4)2 - x(p+1)/4 is the square root.
- This is simplified as (2251 - 1 + 1) /4 = 2519
- Hence, x2519 is the square root iff result.square() == self
Sourcepub const fn relax(&self) -> LooseFieldElement
pub const fn relax(&self) -> LooseFieldElement
Relax a tight field element into a loose one.